/*
 * APP基本JavaScript库，包含以下几个模块
 * 1、本地存储
 * 2、网络请求
 * 3、权限判断
 * 4、界面UI
 * 5、工具辅助
 * 6、窗口操作
 * 7、其它
 */

var SERVER_URL = "https://api.zhuokao360.com/";

/***************************************************
 *                   本地存储                       *
 ***************************************************/

var storage = {
  get: function (key) {
    var value = api.getPrefs({ sync: true, key: key }) || "";

    // 如果JSON字符串无法解析，直接返回
    try {
      return JSON.parse(value);
    } catch (e) {
      return value;
    }
  },

  set: function (key, value) {
    api.setPrefs({
      key: key,
      value: JSON.stringify(value),
    });
  },

  del: function (key) {
    api.removePrefs({
      key: key,
    });
  },
};

/**
 * 获取当前的 professionId
 */
function getProfessionId() {
  return storage.get("profession_id");
}

/**
 * 获取当前的 courseId
 */
function getCourseId() {
  return storage.get("course_id");
}

/**
 * 获取当前 token
 */
function getToken() {
  return storage.get("token");
}

/**
 * 获取当前 keystore
 */
function getKeystore() {
  return storage.get("keystore");
}

/**
 * 获取当前是否登录
 */
function isLogin() {
  var token = getToken();
  return token ? true : false;
}

/***************************************************
 *                   网络请求                       *
 ***************************************************/

/**
 * 封装的请求(使用之前，要引入 bluebird.min.js)
 * @param {String} url 请求地址
 * @param {String} method 异步请求方法类型
 * @param {Object} data POST 数据，method 为 get 时不传
 */
function request(params) {
  var url = params.url.indexOf("http") == 0 ? params.url : SERVER_URL + params.url;
  var method = params.method || "get";
  var data = params.data || {};
  var token = getToken();

  return new Promise(function (resolve, reject) {
    api.ajax(
      {
        url: url,
        method: method,
        headers: {
          "Content-Type": "application/json;charset=utf-8",
          "ZK-TOKEN": token,
          'APP-TYPE': 3
        },
        data: {
          body: data,
        },
      },
      function (ret, err) {
        if (ret) {
          if (ret.code === 200) {
            resolve(ret);
          } else if (ret.code === 600) {
            loginExpired();
          } else if (ret.msg) {
            toast(ret.msg);
          }
          reject(ret);
        } else {
          toast("发生错误,请稍后再试");
          reject(err);
        }
      }
    );
  });
}

/**
 * 登录过期处理逻辑
 */
function loginExpired() {
  toast("登录过期，请重新登录");
  clearUserInfo();
  toLogin();
}

/**
 * 清理用户信息
 */
function clearUserInfo() {
  storage.del("token");
  storage.del("uid");
  storage.del("userinfo");
  storage.del("keystore");
}

/***************************************************
 *                   权限判断                       *
 ***************************************************/

var PERMISSIONNAME = {
  camera: "相机", // 相机/拍照/录像
  contacts: "联系人", // 联系人读取/写入
  microphone: "麦克风", // 使用麦克风录制音频
  photos: "访问相册", // 访问相册|本地存储空间。Android上等同storage权限
  location: "定位", // 定位
  locationAlways: "后台定位", // 后台定位，只支持iOS
  notification: "状态栏通知", // 状态栏通知
  calendar: "日历", // 日历读取/写入。只支持Android
  phone: "电话", // 直接拨打电话/获取手机号码|IMEI。只支持Android
  sensor: "传感器", // 传感器.只支持Android
  sms: "短信", // 读取短信/后台发送短信。只支持Android
  storage: "存储", // 读取/写入|相册|多媒体|本地存储空间。只支持Android
};

/**
 * 判断是否拥有权限
 * @param {Array} perms 权限列表
 */
function hasPermission(perms) {
  return api.hasPermission({
    list: perms,
  });
}

/**
 * 向系统请求权限
 * @param {Array} perms 权限列表
 */
function requestPermission(perms) {
  api.requestPermission({
    list: perms,
    code: 1,
  });
}

/**
 * 确认权限(无权限提示获取)
 * @param {Array} perms 权限列表
 */
function confirmPermission(perms) {
  var resultList = hasPermission(perms);

  // 获取需要请求的权限列表
  var requestList = [];
  resultList.forEach(function (item, index) {
    if (!item || !item.granted) {
      requestList.push(perms[index]);
    }
  });

  // 如果拥有全部权限
  if (requestList.length === 0) {
    return true;
  }

  // 否则，提示请求权限
  var str = "";
  requestList.forEach(function (per) {
    str += PERMISSIONNAME[per] + "、";
  });
  str = str.slice(0, str.length - 1);

  api.confirm(
    {
      title: "提醒",
      msg: "使用相关功能，需要获取 " + str + " 权限\n是否前往设置？",
      buttons: ["去设置", "取消"],
    },
    function (ret) {
      if (ret.buttonIndex === 1) {
        requestPermission(perms);
      }
    }
  );
  return false;
}

/***************************************************
 *                   界面UI                       *
 ***************************************************/

/**
 * 显示加载
 */
function showLoading() {
  api.showProgress();
}

/**
 * 隐藏加载
 */
function hideLoading() {
  api.hideProgress();
}

/**
 * 提示
 * @param {String} msg 提示信息
 */
function toast(msg) {
  api.toast({
    msg: msg,
  });
}

/***************************************************
 *                   工具辅助                       *
 ***************************************************/

function openByWebview(title, url) {
  if (!title) {
    title = "卓考网校";
  }

  api.openWin({
    name: "webview",
    url: "widget://html/webview.html",
    syncCookie: false,
    pageParam: {
      title: title,
      url: url,
    },
    reload: true,
    animation: {
      type: "movein", //动画类型（详见动画类型常量）
      subType: "from_right", //动画子类型（详见动画子类型常量）
      duration: 300, //动画过渡时间，默认300毫秒
    },
  });
}

function openadwebview(title, url) {
  if (!title) {
    title = "详情";
  }
  if (!url) {
    url = WAP_URL;
  }

  api.openWin({
    name: "webview_ad",
    url: "widget://html/adwebview.html",
    pageParam: {
      title: title,
      url: url,
    },
    reload: true,
    animation: {
      type: "movein", //动画类型（详见动画类型常量）
      subType: "from_right", //动画子类型（详见动画子类型常量）
      duration: 300, //动画过渡时间，默认300毫秒
    },
  });
}

/**
 * 打开外部网页
 * @param {String} title 网页标题
 * @param {String} url 网页地址
 */
function webview(title, url) {
  api.openTabLayout({
    name: "webview",
    url: "widget://html/new_webview.html",
    title: title,
    navigationBar: {},
    pageParam: {
      url: url,
    },
  });
}

/**
 * 全局检测是否安装微信
 */
 function WeChat_isGlobalInstalled() {
  var wxPlus = api.require("wxPlus");
  return new Promise ((resolve) => {
    wxPlus.isInstalled(function (ret, err) {
      if (ret.installed) {
        resolve(true);
      } else {
        resolve(false);
      }
    });
  })
}

/**
 * 是否安装微信
 * @param {Function} callback 如果已安装微信,执行的回调函数
 */
function isInstalledWeChat(callback) {
  var wxPlus = api.require("wxPlus");
  wxPlus.isInstalled(function (ret, err) {
    if (ret.installed && typeof callback === "function") {
      callback();
    }
  });
}

/**
 * 打开微信
 * @param {Function} callback 打开微信后执行的回调函数
 */
function openWeChat(callback) {
  isInstalledWeChat(function () {
    window.location.href = "weixin://";

    if (typeof callback === "function") {
      callback();
    }
  });
}

/**
 * 拉起小程序
 * @param {String} path 拉起小程序页面的可带参路径，不填默认拉起小程序首页
 */
function launchMiniProgram(path) {
  isInstalledWeChat(function () {
    var wxPlus = api.require("wxPlus");
    wxPlus.launchMiniProgram({
      miniProgramType: "release",
      userName: "gh_842c06e5db94",
      path: path,
    });
  });
}

/**
 * 复制文本到剪切板
 * @param {String} text 要复制的内容
 * @param {Function} callback 回调函数
 */
function copyText(text, callback) {
  var clipBoard = api.require("clipBoard");
  clipBoard.set(
    {
      value: text,
    },
    function (ret, err) {
      if (ret && typeof callback === "function") {
        callback();
      } else {
        alert("复制出错，请稍后再试");
      }
    }
  );
}

/**
 * 浏览器下载
 * @param {String} url 下载文件地址
 */
function download(url) {
  api.openApp({
    androidPkg: "android.intent.action.VIEW", // 目标应用的包名或 action（Android平台使用），Android下必传
    mimeType: "text/html", // 指定目标应用的响应数据类型，如："text/html"（Android平台使用）
    uri: url, // 指定目标应用响应的uri（Android平台使用）
    iosUrl: url, // 目标应用的url（iOS平台使用），iOS下必传
  });
}

/**
 * 保存图片到系统相册
 * @param {String} path 文件路径
 * @param {Function} callback 回调函数
 */
function saveMediaToAlbum(path, callback) {
  var has = confirmPermission(["photos"]);
  if (!has) {
    return;
  }

  api.saveMediaToAlbum(
    {
      path: path,
    },
    function (ret, err) {
      if (ret && ret.status) {
        toast("保存成功，可到相册查看");

        if (typeof callback === "function") {
          callback();
        }
      } else {
        toast("保存失败");
      }
    }
  );
}

/***************************************************
 *                   窗口操作                       *
 ***************************************************/

/**
 * 关闭当前窗口
 */
function closeWin() {
  api.closeWin();
}

/**
 * 关闭到root页面
 */
function closeToRoot() {
  api.closeToWin({
    name: "root",
  });
}

function login() {
  api.openWin({
    name: "login_smscode",
    url: "widget://html/login_smscode.html",
    slidBackEnabled: true,
    animation: {
      type: "movein", //动画类型（详见动画类型常量）
      subType: "from_bottom", //动画子类型（详见动画子类型常量）
      duration: 300, //动画过渡时间，默认300毫秒
    },
  });
}

function charge() {
  api.openWin({
    name: "charge",
    url: "widget://html/charge.html",
    bounces: false,
  });
}

function reg() {
  api.openWin({
    name: "reg",
    url: "widget://html/reg.html",
    bounces: false,
  });
}

/**
 * 去登录
 */
function toLogin() {
  api.openWin({
    name: "login",
    url: "widget://html/login_smscode.html",
  });
}

/**
 * 打开客服页面
 */
function openChat() {
  api.openTabLayout({
    name: "teacher",
    url: "widget://html/teacher.html",
    title: "班主任",
    navigationBar: {},
  });
}

/**
 * 打开联系会话
 */
function openContactDialog(type, content, tips) {
  api.openFrame({
    name: "contactDialog",
    url: "widget://html/dialog_frame.html",
    rect: {
      x: 0,
      y: 0,
      w: "auto",
      h: "auto",
    },
    pageParam: {
      type: type,
      content: content,
      tips: tips,
    },
  });
}

/**
 * 打开用户协议
 */
function openUserAgreement() {
  webview("用户协议", "https://vip.zhuokao360.com/terms/userAgreement/zkwx.html");
}

/**
 * 打开课程协议
 */
function openCourseAgreement() {
  webview("课程协议", "https://cj.tibaitong.com/notice/relearn");
}

/**
 * 打开隐私政策
 */
function openPrivacyPolicy() {
  webview("隐私政策", "https://vip.zhuokao360.com/terms/privacyPolicy/jmdc.html");
}

/***************************************************
 *                     其它                        *
 ***************************************************/

function init_user_course() {

  var userinfo = JSON.parse(
    api.getPrefs({
      sync: true,
      key: "userinfo",
    })
  );

  var profession_id = api.getPrefs({
    sync: true,
    key: "profession_id",
  });
  if (userinfo) {

    var current_course_id = 1;
    if (userinfo.keystoreMap) {

      if (profession_id == 1) {
        current_course_id = 1;
      } else if (profession_id == 2) {
        if (userinfo.courseId) {
          current_course_id = profession_id + "" + userinfo.courseId;
        } else {
          current_course_id = 23;
        }
      } else if (profession_id == 3) {
        if (userinfo.courseId) {
          current_course_id = profession_id + "" + userinfo.courseId;
        } else {
          current_course_id = 36;
        }
      } else if (profession_id == 4) {
        current_course_id = 412;
      } else if (profession_id == 5) {
        if (userinfo.courseId) {
          current_course_id = profession_id + "" + userinfo.courseId;
        } else {
          current_course_id = 516;
        }
      }
    }

    current_course_id = parseInt(current_course_id);
    if (!current_course_id) current_course_id = 1;

    if (userinfo.keystoreMap) {
      var keystore = userinfo.keystoreMap[current_course_id];
      if (!keystore) keystore = 0;
    } else {
      if (!keystore) keystore = 0;
    }

    api.setPrefs({
      key: "keystore",
      value: keystore,
    });
    api.setPrefs({
      key: "current_course_id",
      value: current_course_id,
    });
  }
}

function iflogin() {
  var token = api.getPrefs({
    sync: true,
    key: "token",
  });
  if (!token) {
    setTimeout(login, 500);
  }
}

function getFooterHeight() {
  return api.safeArea.bottom || 0;
}
